/******************************************************************************/
/* PAlert implementation file.                                                */
/******************************************************************************/
/* This is part of the PolyKit library.                                       */
/* Copyright (C) 1998-1999 by Polycode.                                       */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may be used for free in free- and shareware */
/* programs. For commercial programs, you have to pay a fee to use it. See    */
/* the license file for more information.                                     */
/******************************************************************************/


#define _BUILDING_POLYKIT_LIBRARY_

// PolyKit headers
#include "PDebug.h"
#include "PException.h"
#include "PAlert.h"


/******************************************************************************/
/* String resources                                                           */
/******************************************************************************/
#if __p_os == __p_beos

STRINGRESOURCE_BEGIN(PAlertStrings)
	STRINGRESOURCE_ITEM(IDS_PALERT_ABORT, "Abort")
	STRINGRESOURCE_ITEM(IDS_PALERT_RETRY, "Retry")
	STRINGRESOURCE_ITEM(IDS_PALERT_IGNORE, "Ignore")
	STRINGRESOURCE_ITEM(IDS_PALERT_OK, "Ok")
	STRINGRESOURCE_ITEM(IDS_PALERT_CANCEL, "Cancel")
	STRINGRESOURCE_ITEM(IDS_PALERT_YES, "Yes")
	STRINGRESOURCE_ITEM(IDS_PALERT_NO, "No")
STRINGRESOURCE_END

#endif


/******************************************************************************/
/* Initialize static variables                                                */
/******************************************************************************/
#if __p_os == __p_beos

PMutex PAlert::resLocker;
uint32 PAlert::resCounter = 0;
uint32 PAlert::resID = 0;

#endif



/******************************************************************************/
/* PAlert class                                                               */
/******************************************************************************/

/******************************************************************************/
/* Default constructor                                                        */
/*                                                                            */
/* Input:  "title" is the title of the alert.                                 */
/*         "message" is what you want to write in the alert.                  */
/*         "type" is the type of alert you want.                              */
/*         "buttons" indicates which buttons you want.                        */
/*         "defButton" indicates which button is the default.                 */
/******************************************************************************/
PAlert::PAlert(PString title, PString message, PAlertType type, PAlertButtons buttons, uint8 defButton)
{
	ASSERT(defButton <= 2);

#if __p_os == __p_beos

	// Lock the resource mutex
	resLocker.Lock();

	if (resCounter == 0)
	{
		// It's the first time the alert is constructed, so get the
		// resource id
		resID = AddStringResource(PAlertStrings);
	}

	// Increment the resource counter
	resCounter++;

	// Unlock the mutex
	resLocker.Unlock();

#endif

	// Remember all the arguments
	alertTitle   = title;
	alertMessage = message;
	alertType    = type;
	alertButtons = buttons;
	alertDefault = defButton;
}



/******************************************************************************/
/* Destructor                                                                 */
/******************************************************************************/
PAlert::~PAlert(void)
{
#if __p_os == __p_beos

	// Lock the resource mutex
	resLocker.Lock();

	// Decrement the resource counter
	resCounter--;

	if (resCounter == 0)
	{
		// This is the last alert, so free the string resource id
		RemoveStringResource(resID);
	}

	// Unlock the mutex
	resLocker.Unlock();

#endif
}



/******************************************************************************/
/* Show() will show the alert and first return when the user has pressed one  */
/*      the buttons.                                                          */
/*                                                                            */
/* Output: Is the button ID the user has pressed.                             */
/******************************************************************************/
PAlert::PAlertButton PAlert::Show(void)
{
	PAlertButton button;

#if __p_os == __p_beos

	PString showMsg, but1, but2, but3;
	const char *butAdr1;
	const char *butAdr2;
	const char *butAdr3;
	alert_type type;
	int32 butIndex;
	BAlert *alert;
	BButton *but;

	// Create the message to be shown. This is done because BeOS alerts
	// doesn't have a title bar.
	showMsg = alertTitle + "\n\n" + alertMessage;

	// Find the type of the alert
	switch (alertType)
	{
		case pInfo:
		case pQuestion:
			type = B_INFO_ALERT;
			break;

		case pWarning:
			type = B_WARNING_ALERT;
			break;

		case pStop:
			type = B_STOP_ALERT;
			break;

		default:
			// The alert type isn't implemented
			ASSERT(false);
			type = B_STOP_ALERT;
			break;
	}

	// Find the button strings
	switch (alertButtons)
	{
		case pAbortRetryIgnore:
			but1.LoadString(resID, IDS_PALERT_ABORT);
			but2.LoadString(resID, IDS_PALERT_RETRY);
			but3.LoadString(resID, IDS_PALERT_IGNORE);
			break;

		case pOk:
			but1.LoadString(resID, IDS_PALERT_OK);
			break;

		case pOkCancel:
			but1.LoadString(resID, IDS_PALERT_OK);
			but2.LoadString(resID, IDS_PALERT_CANCEL);
			break;

		case pRetryCancel:
			but1.LoadString(resID, IDS_PALERT_RETRY);
			but2.LoadString(resID, IDS_PALERT_CANCEL);
			break;

		case pYesNo:
			but1.LoadString(resID, IDS_PALERT_YES);
			but2.LoadString(resID, IDS_PALERT_NO);
			break;

		case pYesNoCancel:
			but1.LoadString(resID, IDS_PALERT_YES);
			but2.LoadString(resID, IDS_PALERT_NO);
			but3.LoadString(resID, IDS_PALERT_CANCEL);
			break;

		default:
			// The alert buttons isn't implemented
			ASSERT(false);
			break;
	}

	// Convert the strings into pointers
	butAdr1 = but1;
	butAdr2 = (but2.IsEmpty() ? NULL : (const char *)but2);
	butAdr3 = (but3.IsEmpty() ? NULL : (const char *)but3);

	// Create the alert
	alert = new BAlert(NULL, showMsg, butAdr1, butAdr2, butAdr3, B_WIDTH_AS_USUAL, type);
	if (alert == NULL)
		throw PMemoryException();

	// Set the default button and keys
	switch (alertButtons)
	{
		case pAbortRetryIgnore:
			// Remove the default button
			but = alert->ButtonAt(2);
			but->MakeDefault(false);

			// And set a new default button
			but = alert->ButtonAt(alertDefault);
			but->MakeDefault(true);
			break;

		case pOk:
			break;

		case pOkCancel:
		case pRetryCancel:
		case pYesNo:
			// Make sure that the default is a valid button
			if (alertDefault > 1)
				alertDefault = 0;

			// Remove the default button
			but = alert->ButtonAt(1);
			but->MakeDefault(false);

			// And set a new default button
			but = alert->ButtonAt(alertDefault);
			but->MakeDefault(true);

			// Escape can be pressed to indicate cancel
			alert->SetShortcut(1, B_ESCAPE);
			break;

		case pYesNoCancel:
			// Remove the default button
			but = alert->ButtonAt(2);
			but->MakeDefault(false);

			// And set a new default button
			but = alert->ButtonAt(alertDefault);
			but->MakeDefault(true);

			// Escape can be pressed to indicate cancel
			alert->SetShortcut(2, B_ESCAPE);
			break;

		default:
			// The alert buttons isn't implemented
			ASSERT(false);
			break;
	}

	// Show the alert
	butIndex = alert->Go();

	// Find out which button that has been pressed
	switch (alertButtons)
	{
		case pAbortRetryIgnore:
			button = (butIndex == 0 ? pIDAbort : (butIndex == 1 ? pIDRetry : pIDIgnore));
			break;

		case pOk:
			button = pIDOk;
			break;

		case pOkCancel:
			button = (butIndex == 0 ? pIDOk : pIDCancel);
			break;

		case pRetryCancel:
			button = (butIndex == 0 ? pIDRetry : pIDCancel);
			break;

		case pYesNo:
			button = (butIndex == 0 ? pIDYes : pIDNo);
			break;

		case pYesNoCancel:
			button = (butIndex == 0 ? pIDYes : (butIndex == 1 ? pIDNo : pIDCancel));
			break;

		default:
			// The alert buttons isn't implemented
			ASSERT(false);
			button = pIDCancel;
			break;
	}

#elif __p_os == __p_windows

	UINT type;
	int result;

	// Use the appropiate icon for the alert type
	switch (alertType)
	{
		case pInfo:
			type = MB_ICONINFORMATION;
			break;

		case pWarning:
			type = MB_ICONEXCLAMATION;
			break;

		case pStop:
			type = MB_ICONSTOP;
			break;

		default:
			// The alert type isn't implemented
			ASSERT(false);
			break;
	}

	// Merge the buttons type
	switch (alertButtons)
	{
		case pAbortRetryIgnore:
			type |= MB_ABORTRETRYIGNORE;
			break;

		case pOk:
			type |= MB_OK;
			break;

		case pOkCancel:
			type |= MB_OKCANCEL;
			break;

		case pRetryCancel:
			type |= MB_RETRYCANCEL;
			break;

		case pYesNo:
			type |= MB_YESNO;
			break;

		case pYesNoCancel:
			type |= MB_YESNOCANCEL;
			break;

		default:
			// The alert buttons isn't implemented
			ASSERT(false);
			break;
	}

	// Merge the default button type
	switch (alertDefault)
	{
		case 0:
			type |= MB_DEFBUTTON1;
			break;

		case 1:
			type |= MB_DEFBUTTON2;
			break;

		case 2:
			type |= MB_DEFBUTTON3;
			break;
	}

	// Show the alert
	result = MessageBox(NULL, alertMessage, alertTitle, type);

	// Find out which button that has been pressed
	switch (result)
	{
		case IDABORT:
			button = pIDAbort;
			break;

		case IDCANCEL:
			button = pIDCancel;
			break;

		case IDIGNORE:
			button = pIDIgnore;
			break;

		case IDNO:
			button = pIDNo;
			break;

		case IDOK:
			button = pIDOk;
			break;

		case IDRETRY:
			button = pIDRetry;
			break;

		case IDYES:
			button = pIDYes;
			break;

		default:
			// The alert buttons isn't implemented
			ASSERT(false);
			break;
	}

#elif __p_os == __p_linux
implement
#endif

	return (button);
}
