/******************************************************************************/
/* PString header file.                                                       */
/******************************************************************************/
/* This is part of the PolyKit library.                                       */
/* Copyright (C) 1998-1999 by Polycode.                                       */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may be used for free in free- and shareware */
/* programs. For commercial programs, you have to pay a fee to use it. See    */
/* the license file for more information.                                     */
/******************************************************************************/


#ifndef __PString_h
#define __PString_h

// PolyKit headers
#include "POS.h"
#include "PDebug.h"
#include "ImportExport.h"


/******************************************************************************/
/* String resource macros                                                     */
/******************************************************************************/
typedef struct PStringResource
{
	int32	strNum;
	char *	string;
} PStringResource;

#define STRINGRESOURCE_BEGIN(name) static PStringResource name[] = {
#define STRINGRESOURCE_END {-1, NULL} };

#define STRINGRESOURCE_ITEM(strNum, string) {strNum, string},


extern char *pEmptyString;


/******************************************************************************/
/* Global functions                                                           */
/******************************************************************************/
#if __p_os == __p_beos && __POWERPC__
#pragma export on
#endif

_IMPEXP_PKLIB uint32 AddStringResource(PStringResource *resource);
_IMPEXP_PKLIB void RemoveStringResource(uint32 resID);



/******************************************************************************/
/* PString class                                                              */
/******************************************************************************/
class _IMPEXP_PKLIB PString
{
public:
	PString(void);
	PString(const PString &string);
	PString(const char *string);
	~PString(void);				// Do NOT make this virtual!!!

	int32 GetLength(void) const;
	int32 GetByteLength(void) const;
	char GetAt(int32 index) const;

	bool IsEmpty(void) const;
	void MakeEmpty(void);

	void MakeLower(void);
	void MakeUpper(void);

	void Format(const char *formatString, ...);
	void Format(uint32 resID, int32 strNum, ...);
	void LoadString(uint32 resID, int32 strNum);

	int32 Find(char ch, int32 startPos = -1) const;
	int32 Find(const char *subString, int32 startPos = -1) const;
	int32 ReverseFind(char ch) const;

	PString Left(int32 count) const;
	PString Mid(int32 first) const;
	PString Mid(int32 first, int32 count) const;
	PString Right(int32 count) const;

	void TrimLeft(void);
	void TrimRight(void);

	PString MakeNumber(void) const;
	PString StringReplace(const char *search, const char *replace) const;

	int32 CompareNoCase(const char *string) const;

	char *GetBuffer(int32 minBufLength);
	char *GetBufferSetLength(int32 newLength);
	void ReleaseBuffer(int32 newLength = -1);
	void FreeExtra(void);

	operator const char * () const;

	char operator [] (int32 index) const;

	const PString & operator = (const PString &string);
	const PString & operator = (const char *string);

	_IMPEXP_PKLIB friend PString operator + (const PString &string1, const PString &string2);
	_IMPEXP_PKLIB friend PString operator + (const PString &string1, const char *string2);
	_IMPEXP_PKLIB friend PString operator + (const char *string1, const PString &string2);

	const PString & operator += (const PString &string);
	const PString & operator += (const char *string);

	friend inline bool operator == (const PString &string1, const PString &string2) { return ((strcmp(string1.stringData, string2.stringData) == 0 ? true : false)); };
	friend inline bool operator == (const PString &string1, const char *string2) { return ((strcmp(string1.stringData, string2) == 0 ? true : false)); };
	friend inline bool operator == (const char *string1, PString &string2) { return ((strcmp(string1, string2.stringData) == 0 ? true : false)); };

	friend inline bool operator != (const PString &string1, const PString &string2) { return ((strcmp(string1.stringData, string2.stringData) != 0 ? true : false)); };
	friend inline bool operator != (const PString &string1, const char *string2) { return ((strcmp(string1.stringData, string2) != 0 ? true : false)); };
	friend inline bool operator != (const char *string1, PString &string2) { return ((strcmp(string1, string2.stringData) != 0 ? true : false)); };

	friend inline bool operator < (const PString &string1, const PString &string2) { return ((strcmp(string1.stringData, string2.stringData) < 0 ? true : false)); };
	friend inline bool operator < (const PString &string1, const char *string2) { return ((strcmp(string1.stringData, string2) < 0 ? true : false)); };
	friend inline bool operator < (const char *string1, PString &string2) { return ((strcmp(string1, string2.stringData) < 0 ? true : false)); };

	friend inline bool operator > (const PString &string1, const PString &string2) { return ((strcmp(string1.stringData, string2.stringData) > 0 ? true : false)); };
	friend inline bool operator > (const PString &string1, const char *string2) { return ((strcmp(string1.stringData, string2) > 0 ? true : false)); };
	friend inline bool operator > (const char *string1, PString &string2) { return ((strcmp(string1, string2.stringData) > 0 ? true : false)); };

	friend inline bool operator <= (const PString &string1, const PString &string2) { return ((strcmp(string1.stringData, string2.stringData) <= 0 ? true : false)); };
	friend inline bool operator <= (const PString &string1, const char *string2) { return ((strcmp(string1.stringData, string2) <= 0 ? true : false)); };
	friend inline bool operator <= (const char *string1, PString &string2) { return ((strcmp(string1, string2.stringData) <= 0 ? true : false)); };

	friend inline bool operator >= (const PString &string1, const PString &string2) { return ((strcmp(string1.stringData, string2.stringData) >= 0 ? true : false)); };
	friend inline bool operator >= (const PString &string1, const char *string2) { return ((strcmp(string1.stringData, string2) >= 0 ? true : false)); };
	friend inline bool operator >= (const char *string1, PString &string2) { return ((strcmp(string1, string2.stringData) >= 0 ? true : false)); };

protected:
	typedef struct PStringData
	{
		int32	refCount;
		int32	stringLen;
		int32	allocatedLen;
	} PStringData;

	void AllocBuffer(int32 length);
	void Release(void);
	void Release(PStringData *data);

	int32 CountLength(void);

	void CopyBeforeWrite(void);
	void AllocBeforeWrite(int32 strLen);
	void AssignCopy(const char *string);
	void AllocCopy(PString &dest, int32 copyIndex, int32 copyLen, int32 extraLen) const;

	void ConcatInPlace(const char *string);
	void ConcatCopy(const char *sourceString, int32 sourceLen, const char *appendString, int32 appendLen);

	void FormatIt(const char *formatString, va_list argList);

	inline void Init(void) { stringData = pEmptyString; };
	inline PStringData *GetData(void) const { return ((PStringData *)(stringData - sizeof(PStringData))); };

#if __p_os == __p_beos
	inline int32 GetCharLength(const char byte) const { return (((0xE5000000 >> ((byte >> 3) & 0x1E) & 3)) + 1); };
#else
	inline int32 GetCharLength(const char byte) const { return (1); };
#endif

	char *stringData;
};

#if __p_os == __p_beos && __POWERPC__
#pragma export off
#endif

#endif
