/******************************************************************************/
/* PTimer implementation file.                                                */
/******************************************************************************/
/* This is part of the PolyKit library.                                       */
/* Copyright (C) 1998-1999 by Polycode.                                       */
/* All rights reserved.                                                       */
/*                                                                            */
/* This source, or parts thereof, may be used for free in free- and shareware */
/* programs. For commercial programs, you have to pay a fee to use it. See    */
/* the license file for more information.                                     */
/******************************************************************************/


#define _BUILDING_POLYKIT_LIBRARY_

// PolyKit headers
#include "PException.h"
#include "PString.h"
#include "PSynchronize.h"
#include "PThread.h"
#include "PTimer.h"


/******************************************************************************/
/* PTimer class                                                               */
/******************************************************************************/

/******************************************************************************/
/* Constructor                                                                */
/*                                                                            */
/* Input:  "looper" is a pointer to the looper to send the message to.        */
/*         "id" is the timer id.                                              */
/******************************************************************************/
#if __p_os == __p_beos

PTimer::PTimer(BLooper *looper, uint16 id)
{
	// Initialize member variables
	running    = false;
	paused     = false;

	sendLooper = looper;
	sendId     = id;
	waitTime   = 1;

	// Allocate the needed semaphores
	runSem = new PSemaphore(0, true);
	if (runSem == NULL)
		throw PMemoryException();

	timeoutSem = new PSemaphore(1, true);
	if (timeoutSem == NULL)
	{
		delete runSem;
		throw PMemoryException();
	}

	// Allocate the thread object
	thread = new PThread;
	if (thread == NULL)
	{
		delete timeoutSem;
		delete runSem;
		throw PMemoryException();
	}

	// Initialize the thread
	PString name;

	name.Format("PTimer (%d)", id);
	thread->SetName(name);
	thread->SetHookFunc(ThreadFunc, this);
	thread->StartThread();
}



/******************************************************************************/
/* Constructor                                                                */
/*                                                                            */
/* Input:  "window" is a handle to a window, the message will be send to.     */
/*         "id" is the timer id.                                              */
/******************************************************************************/
#elif __p_os == __p_windows

PTimer::PTimer(HWND window, uint16 id)
{
	// Initialize member variables
	windowHandle = window;
	sendId		 = id;
	waitTime	 = 1;
	running		 = false;
}

#endif



/******************************************************************************/
/* Destructor                                                                 */
/******************************************************************************/
PTimer::~PTimer(void)
{
#if __p_os == __p_beos

	// Remove the thread
	paused = false;
	timeoutSem->Unlock();		// So the semaphore can be locked in the thread
	runSem->Unlock();			// Wakeup the thread

	// Wait for the thread to finish
	thread->WaitOnThread();

	// Cleanup
	delete thread;
	delete runSem;
	delete timeoutSem;

#elif __p_os == __p_windows

	StopTimer();

#endif
}



/******************************************************************************/
/* SetElapseValue() will change the time to elapse before a message is sent.  */
/*                                                                            */
/* Input:  "value" is the number to wait in milliseconds.                     */
/******************************************************************************/
void PTimer::SetElapseValue(uint32 value)
{
	waitTime = value;
}



/******************************************************************************/
/* StartTimer() will start the timer.                                         */
/******************************************************************************/
void PTimer::StartTimer(void)
{
#if __p_os == __p_beos

	// Start the timer
	if (!running)
	{
		paused = false;
		timeoutSem->Lock();

		runSem->Unlock();
		running = true;
	}

#elif __p_os == __p_windows

	// Start running the timer if it has not already started
	if (!running)
	{
		VERIFY(::SetTimer(windowHandle, sendId, waitTime, NULL) != 0);
		running = true;
	}

#endif
}



/******************************************************************************/
/* StopTimer() will stop the timer.                                           */
/******************************************************************************/
void PTimer::StopTimer(void)
{
#if __p_os == __p_beos

	// Stop the timer
	if (running)
	{
		runSem->Lock();
		running = false;

		paused = true;
		timeoutSem->Unlock();
	}

#elif __p_os == __p_windows

	// Stop the timer if it is running
	if (running)
	{
		::KillTimer(windowHandle, sendId);
		running = false;
	}

#endif
}



/******************************************************************************/
/* ThreadFunc() is the main timer thread.                                     */
/*                                                                            */
/* Input:  "object" is a pointer to the current timer object.                 */
/*                                                                            */
/* Output: Always 0.                                                          */
/******************************************************************************/
#if __p_os == __p_beos

int32 PTimer::ThreadFunc(void *object)
{
	PTimer *obj = (PTimer *)object;
	PSyncError retVal;

	do
	{
		// If we can get this semaphore, the timer is started
		retVal = obj->runSem->Lock();
		if (retVal == pSyncOk)
		{
			// Release it again
			obj->runSem->Unlock();

			// This semaphore is locked, so we will never get it, which
			// means, we will get a timeout after the user specific
			// amount of time. If we get the semaphore anyway, it means
			// we have to exit the thread.
			retVal = obj->timeoutSem->Lock(obj->waitTime);
			if (retVal == pSyncTimeout)
			{
				// Send the message
				BMessage msg(PM_TIMER);

				msg.AddInt16("ID", obj->sendId);
				obj->sendLooper->PostMessage(&msg);

				retVal = pSyncOk;
			}
			else
			{
				if (retVal == pSyncOk)
				{
					// We got the timeout semaphore, check to see if we have to exit
					if (!obj->paused)
						break;		// Exit the thread
					else
						obj->timeoutSem->Unlock();	// Release the semaphore again
				}
			}
		}
	}
	while (retVal == pSyncOk);

	return (0);
}

#endif

#if __p_os == __p_linux
implement all
#endif
